<?php
/* @wordpress-plugin
 * Plugin Name:       WooCommerce Royal Mail, Evri, & DPD Shipping Calculator Pro
 * Plugin URI:        https://wpruby.com/plugin/woocommerce-royal-mail-shipping-calculator-pro/
 * Description:       Add Royal Mail, Evri, DPD shipping prices to your WooCommerce store.
 * Version:           4.1.1
 * WC requires at least: 3.0
 * WC tested up to: 6.3
 * Author:            WPRuby
 * Author URI:        https://wpruby.com
 * Text Domain:       wc-royal-mail
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Domain Path:       /languages
 */

define('ROYALMAILPRO_VERSION', '4.1.1');
define('ROYALMAILPRO_URL', plugin_dir_url(__FILE__));
define('ROYALMAILPRO_DIR', plugin_dir_path(__FILE__));
define('WPRUBY_ROYALMAIL_STORE_URL', 'https://wpruby.com/edd_sl/woocommerce-royal-mail' );
define("WPRUBY_EVRI_PLUGIN_ID", "wpruby-evri");
define("WPRUBY_DPD_UK_PLUGIN_ID", "wpruby-dpd-uk");
define( 'WPRUBY_ROYALMAIL_ITEM_ID', 12477);

class WPRuby_Royal_Mail{

	/**
	 * The single instance of the class.
	 *
	 * @var WPRuby_Royal_Mail
	 * @since 1.4.0
	 */
	protected static $_instance = null;

	/** @var array **/
	private $active_plugins;

	/**
	 * WPRuby_Royal_Mail constructor.
	 */
	public function __construct() {

		$this->load_dependencies();
		add_action('wp_loaded', [$this, 'license_handler']);
		add_action('admin_init', [$this, 'plugin_updater']);

		$this->active_plugins = apply_filters('active_plugins', (array) get_option( 'active_plugins', [] ));
		add_action( 'admin_init', [$this, 'deactivate_lite_version'] );

		if ($this->is_woocommerce_active()) {
			add_filter('woocommerce_shipping_methods', [$this, 'add_methods']);
			add_filter('woocommerce_shipping_init', [$this, 'init_shipping_methods']);
			add_filter('woocommerce_product_options_shipping', [$this, 'add_dropshipping_field']);
			add_filter('woocommerce_process_product_meta', [$this, 'dropshipping_field_save']);
		}
	}

	/**
	 * @param $post_id
	 */
	public function dropshipping_field_save( $post_id ){
		$woocommerce_text_field = (isset($_POST['_dropshipping_postcode']))?$_POST['_dropshipping_postcode']:'';
		update_post_meta( $post_id, '_dropshipping_postcode', esc_attr( $woocommerce_text_field ) );
	}

	/**
	 * Add dropshipping field in product page.
	 */
	public function add_dropshipping_field(){
		woocommerce_wp_text_input( array(
			'id'                => '_dropshipping_postcode',
			'label'             => __( 'Dropshipper Postcode', 'woocommerce' ),
			'desc_tip'          => true,
			'description'       => __( 'The shipping cost of this product will be calculated based on this postcode, leave it empty to use your shop postcode.', 'woocommerce' ),
			'type'              => 'text'
		) );
	}

	/**
	 * Load the method class
	 */
	public function init_shipping_methods()
	{
		require_once plugin_dir_path( __FILE__ ) . 'shared/class-wpruby-shipping-method.php';
		require_once plugin_dir_path( __FILE__ ) . 'shared/class-wpruby-abstract-calculator.php';
		require_once plugin_dir_path( __FILE__ ) . 'royalmail/class-wc-royal-mail-shipping-method.php';
		require_once plugin_dir_path(__FILE__) . 'evri/includes/class-wpruby-evri-domestic-calculator.php';
		require_once plugin_dir_path(__FILE__) . 'evri/includes/class-wpruby-evri-intl-calculator.php';
		require_once plugin_dir_path(__FILE__) . 'evri/class-evri-shipping.php';
		require_once plugin_dir_path(__FILE__) . 'dpd/class-dpd-calculator.php';
		require_once plugin_dir_path(__FILE__) . 'dpd/class-dpd-uk-shipping.php';
	}

	/**
	 * @param $methods
	 *
	 * @return array
	 */
	public function add_methods( $methods )
	{
		$methods['wpruby_royalmail'] = 'WC_Royal_Mail_Shipping_Method';
		$methods[WPRUBY_EVRI_PLUGIN_ID] = 'Evri_Shipping';
		$methods[WPRUBY_DPD_UK_PLUGIN_ID] = 'DPD_UK_Shipping';
		return $methods;
	}

	/**
	 * deactivate the lite version
	 */
	public function deactivate_lite_version() {
		if (in_array('royal-mail-woocommerce-shipping-calculator/wc-royal-mail.php', $this->active_plugins)) {
			deactivate_plugins( 'royal-mail-woocommerce-shipping-calculator/wc-royal-mail.php' );
		}
	}


	/**
	 * @return bool
	 */
	private function is_woocommerce_active(){
		if ( is_multisite() ){
			$this->active_plugins = array_merge( $this->active_plugins, get_site_option( 'active_sitewide_plugins', [] ) );
		}

		return in_array( 'woocommerce/woocommerce.php', $this->active_plugins ) || array_key_exists( 'woocommerce/woocommerce.php', $this->active_plugins );
	}

	public function plugin_updater()
	{
		$license_key = trim( get_option( 'wpruby_royalmail_license_key' ) );
		new EDD_RoyalMail_Plugin_Updater( WPRUBY_ROYALMAIL_STORE_URL, __FILE__, [
			'version' 	=> ROYALMAILPRO_VERSION,		// current version number
			'license' 	=> $license_key,	// license key (used get_option above to retrieve from DB)
			'item_id'     => WPRUBY_ROYALMAIL_ITEM_ID,	// name of this plugin
			'author' 	=> 'Waseem Senjer',	// author of this plugin
			'url'           => home_url()
		]);
	}

	/**
	 * License Handler
	 */
	public function license_handler()
	{
		$license_handler = new WPRuby_RoyalMail_Licence_Handler('wpruby_royalmail_license_key');
		$license_handler->setPage('wc-settings');
		$license_handler->setSection('wc_royal_mail_shipping_method');
		$license_handler->setReturnUrl(admin_url('admin.php?page=wc-settings&tab=shipping'));
		$license_handler->setPluginName('WooCommerce Royal Mail Shipping Calculator Pro');
	}

	private function load_dependencies()
	{
		// Updates and Licence Handling
		if( !class_exists( 'EDD_RoyalMail_Plugin_Updater' ) ) {
			include( dirname( __FILE__ ) . '/shared/EDD_RoyalMail_Plugin_Updater.php' );
		}
		if(!class_exists( 'WPRuby_RoyalMail_Licence_Handler' )){
			include( dirname( __FILE__ ) . '/shared/WPRuby_RoyalMail_Licence_Handler.php' );
		}
	}

	/**
	 * @return WPRuby_Royal_Mail
	 */
	public static function get_instance(){
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}
}

/** initiate the plugin */
WPRuby_Royal_Mail::get_instance();
